/*
 * display.c -- displays data and status banners
 * Part of the tcpick project
 *
 * Author: Francesco Stablum <duskdruid @ despammed.com>
 *
 * Copyright (C) 2003, 2004  Francesco Stablum
 * Licensed under the GPL
 *
 */

/*
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* FIXME: all this code seems to be written by a yeti */

#include "tcpick.h"
#include "extern.h"

char * 
status2str( Status_t status )
/* converts the status flag to a string */
{
	char *str = (char *)S_calloc(1,128);
	switch(status) {
	case SYN_SENT:
		strcpy( str, "SYN-SENT" );
		break;
	case SYN_RECEIVED:
		strcpy( str, "SYN-RECEIVED" );
		break;
	case ESTABLISHED:
		strcpy( str, "ESTABLISHED" );
		break;
	case FIN_WAIT_1:
		strcpy( str, "FIN-WAIT-1" );
		break;
	case FIN_WAIT_2__CLOSE_WAIT:
		strcpy( str, "FIN-WAIT-2" );
		/* should it be CLOSE-WAIT? */
		break;
	case TIME_WAIT__LAST_ACK:
		strcpy( str, "TIME-WAIT" );
		/* should it be LAST-ACK? */
		break;
	case CLOSED:
		strcpy( str, "CLOSED" );
		/* should it be TIME-WAIT? */
		break;
	default:
		strcpy( str, "ERROR" );
		break;
	}
	
	return str;
}

int 
display_status( FILE * out, conn_t * conn, Status_t status )
/* display a "status banner" */
{
	char *client_name;
	char *server_name;
	char *s_time = (char *)S_calloc( 128, 1 );
	char *status_string = (char *)S_calloc( 1, 128 );
	
	if ( flags.notstatus ) /* FIXME: sucks!
				  and what if out will be a file? */
		return 0;
	client_name=(char *)cache_getnamebyip(conn->client.ip);
	server_name=(char *)cache_getnamebyip(conn->server.ip);
	
	get_numbered_time( s_time );
	Color( c_TIME, out,"%-16s ",s_time );
	Color( c_NUMBERING, out,"%-6d ",conn->num );
	
	status_string = status2str( status );
	
	Color( _STATUS_COLOR(status),
		     out,"%-14s",status_string );
	
	S_free( status_string );
	
	Color( c_CLIENTNAME_STATUS, out, " %s:%s ",
		     client_name,
		     getportname(conn->client.port) );
	
	Color( _STATUS_COLOR(status), out,"> " );
	
	Color( c_SERVERNAME_STATUS, out,"%s",
		     server_name );
	
	fprintf( out,":" );
		
	Color( c_SERVICE, out,"%s",
		     getportname(conn->server.port) );
	
	fprintf( out, "\n" );
	fflush( out );
}
int display_header( FILE * out )
/* displays a banner for the header of the packet sniffed */
{ /* FIXME: sucks */
	char * source = NULL;
	char * dest   = NULL;
	char * s_time = (char *)S_calloc(128,1);
	source=(char *)cache_getnamebyip(ippacket->ip_src.s_addr);
	Debug("[display_header]: source(%x)",source);
	Debug("[display_header]: source=%s",source);
	get_numbered_time(s_time);
	Color ( c_TIME, out,"%-16s ",s_time );

	Color( c_CLIENTNAME_HEADER, out,"%s",source );
  
	fprintf(out,":");

	Color( c_SERVICE_HEADER, out,"%s ",getportname(tcppacket->source) );

	if(tcppacket->urg)
		Color(c_FLAGS, out,"U");
	if(tcppacket->ack)
		Color(c_FLAGS, out,"A");
	if(tcppacket->psh)
		Color(c_FLAGS, out,"P");
	if(tcppacket->rst)
		Color(c_FLAGS, out,"R");
	if(tcppacket->syn)
		Color(c_FLAGS, out,"S");
	if(tcppacket->fin)
		Color(c_FLAGS, out,"F");

  
	dest = (char *)cache_getnamebyip( ippacket->ip_dst.s_addr );

	Color(c_DIRECTION, out," > ");

	Color(c_SERVERNAME_HEADER, out,"%s",dest);

	fprintf(out,":");

	Color(c_SERVICE_HEADER, out,"%s",getportname(tcppacket->dest));

	fprintf(out," (%i)\n",payload_len);

	fflush(out);

}

int
out_h( FILE * out, u_char * buf, int buflen )
/* hex-spaced mode */
{
#define CHAR (int)( * buf )
	while ( buflen ) {
		fprintf(out,"%2.2x ", CHAR);
		buf++;
		buflen--;
	}
	if ( DISPLAY_NL )
		fputc( '\n' , out );
}

int
out_hn( FILE * out, u_char * buf, int buflen )
/* non printable carachters are displayed as <hex> */
{
#define CHAR (int)( * buf )
	while ( buflen ) {
		if( ( isascii( CHAR ) && !iscntrl( CHAR ) ) || ( CHAR == '\n' ) )
			fputc( CHAR , out );
		else
			Color(c_NONPRINT, out, "<%2.2x>", CHAR);
		buf++;
		buflen--;
	}
	if ( * ( --buf ) != '\n' )
		if ( DISPLAY_NL )
			fputc( '\n' , out );
}

int
out_p( FILE * out, u_char * buf, int buflen )
/* non printable carachters are displayed as dots */
{
#define CHAR (int)( * buf )
	while (buflen) {
		if( ( isascii( CHAR ) && !iscntrl( CHAR ) ) || ( CHAR == '\n' ) )
			fputc( CHAR , out );
		else
			Color( c_NONPRINT, out, "." );
		buf++;
		buflen--;
	}
	if ( * ( --buf ) != '\n' )
		if ( DISPLAY_NL )
			fputc( '\n' , out );
}

/* WARNING: 
   if you want to modify out_xa, see carefully if it 
   affects out_x too */

#define CURR_C (* ( buf + pos ) )
#define POS_FROM_NL ( pos % 16 ) /* 0-15 */
#define UNPRINT_CHAR '.'
#define STRING_SPACES 7
#define MISSING_CHARS (16 - ( pos % 16 ) )
#define IS_NL (! ( ( pos + 1 ) % 16 ) )
#define CALCULATE_SPACES_NUM ( STRING_SPACES + ( MISSING_CHARS * 2 ) + ( MISSING_CHARS / 2 ) - \
			       ( ( MISSING_CHARS % 2 == 0 )? 1 : 0 ) )
int
out_xa (FILE * out, u_char * buf, int buflen)
/* hexdump with ascii */
{ /* FIXME: needs explanation */
	int pos;
	char str[17] = "..............";
	int i;
	for(pos = 0; pos < buflen; pos++) {
		if (! POS_FROM_NL ) {
			if ( pos )
				Color(c_HEXOFF, out,"0x%-8.4x", pos);
			else
				Color(c_HEXOFF0, out,"0x%-8.4x", pos);
		}
		fprintf(out, 
			pos % 2 ? "%2.2x ": "%2.2x", 
			CURR_C );
		
		str[POS_FROM_NL] = CURR_C;

		if ( IS_NL ) {
			for (i = 1; i < STRING_SPACES ; i++ )
				fprintf( out, " " );

			for (i = 1; i <= 16 ; i++ ) {
				if ( isgraph( str[i] ) )
					fputc( str[i], out );
				else
					Color( c_NONPRINT, out, "." );
			}
			fprintf( out, "\n" );
		}
	}
	if ( pos % 16 ) {
		for (i = 1; i <= CALCULATE_SPACES_NUM ; i++ )
			fprintf( out, " ", i%10 );
		for (i = 1; i <= POS_FROM_NL ; i++ ) {
			if ( isgraph( str[i] ) )
				fputc( str[i], out );
			else
				Color( c_NONPRINT, out, "." );
		}
		fprintf( out, "\n");
	}

}

int
out_x (FILE * out, u_char * buf, int buflen)
/* hexdump */
{
	int pos;

	for(pos = 0; pos < buflen; pos++) {
		if ( ! ( pos % 16 ) )
			if (! POS_FROM_NL ) {
				if ( pos )
					Color(c_HEXOFF, out,"0x%-8.4x", pos);
				else
					Color(c_HEXOFF0, out,"0x%-8.4x", pos);
			}
		fprintf(out, 
			pos % 2 ? "%2.2x ": "%2.2x", 
			* ( buf + pos ) );
		if ( IS_NL )
			fprintf(out, "\n");

	}
	fprintf(out, "\n");
}
