/*
 * tcpick.c -- main project file
 * Part of the tcpick project
 *
 * Author: Francesco Stablum <duskdruid @ despammed.com>
 *
 * Copyright (C) 2003, 2004  Francesco Stablum
 * Licensed under the GPL
 *
 */

/* 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at you option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111,
 * USA.
 *
 * Please read the COPYING file for the license details!
 */

/*
 * tcpick project needs somebody suggesting me bugs and new features!
 * If you want to contribute sending patches, finding bugs, compilation
 * errors, platform-specific incompatibilies you are invited to the tcpick
 * mailing-list:
 *
 * email address: 
 *   <tcpick-project@lists.sourceforge.net>
 * Archive:
 *   http://sourceforge.net/mailarchive/forum.php?forum=tcpick-project
 * Subscribe:
 *   http://lists.sourceforge.net/lists/listinfo/tcpick-project
 */

/*
 * in the code you will find many (sooo many) FIXME's: it means that
 * I (or somebody else) should improve the code.
 */

#include "tcpick.h"
#include "globals.h"

struct flags_t flags;

char *errbuf[PCAP_ERRBUF_SIZE];
struct bpf_program filter_compiled;
bpf_u_int32 netp; /* ip */
bpf_u_int32 maskp; /* subnet mask */
struct in_addr addr;
char *other_args = NULL;
pcap_t *descr;
const u_char *packet;
u_char *curr_char;
int curr_pos = 0;
struct pcap_pkthdr hdr;

extern void got_packet();

int main(int argc, char **argv) 
{
	parse_args(argc, argv);
	
	Message( 1, c_WELCOME, stdout,
		 "Starting " PACKAGE_STRING " on %s", get_actual_time() );
	//a newline carachter is in get_actual_time() string
	
	if ( flags.trackfirst > -1 ) {
		Message( 1, c_SETTING, stdout, 
			 "Number of connections that will be tracked: %i",
			 flags.trackfirst );
	}

	if ( getuid() ) {
		Error( "you should be *root* to run %s", PACKAGE_STRING );
	}

	/* making the filter working */
	if ( dev == NULL && readfile == NULL ) {
		dev = pcap_lookupdev( (char*) errbuf );
	}

	if ( dev == NULL && readfile == NULL ) {
		Error( errbuf );
		exit( TCPICK_FAILURE );
	}

	if ( readfile )
		Message( 1, c_INTERFACE, stdout, "%s: reading from %s",exe,readfile );
	else
		Message( 1, c_INTERFACE, stdout, "%s: listening on %s",exe,dev );

	if ( dev ) {
		/* ask pcap for the network address and mask of the device */
		ret = pcap_lookupnet( dev, &netp,
				      &maskp, (char *)errbuf);
		if ( ret == -1 ) {
			netp = 0;
			maskp = 0;
			Error( "%s", errbuf );
		}

		addr.s_addr = netp;
		net = (char *)strdup( inet_ntoa(addr) );
		if ( net == NULL ) {
			perror ( "inet_ntoa" );
			bug();
		}

		addr.s_addr = maskp;
		mask = (char *)strdup( inet_ntoa(addr) );
		if (mask == NULL) {
			perror( "inet_ntoa" );
			bug();
		}
	}

	if ( dev && readfile ) {
		Error( "Please specify either a file (with -r <filename>\n)"
			 "\t or an interface (with -i <int>), not both" );
		exit( TCPICK_FAILURE );
	}

	if (! readfile )
		descr = pcap_open_live( dev, BUFSIZ, flags.notpromisc ? 0 : 1 , -1, (char *)errbuf );
	else
		descr = pcap_open_offline( readfile, (char *)errbuf );

	if (! descr ) {
		Error( "%s", errbuf );
		exit( TCPICK_FAILURE );
	}

	/* compiling the filter */
	if ( filter ) {
		Message (1, c_SETTING, stdout, "setting filter: \"%s\"",filter);
		
		if( ( pcap_compile( descr, &filter_compiled, filter, 0, (int)net ) == -1) ) {
			Error("error compiling filter \"%s\"",filter);
		}
		pcap_setfilter( descr, &filter_compiled );
	}

/* FIXME: next lines SUCKS */
	dl_id = pcap_datalink( descr );
	dl_str = (char *)dl2str( dl_id );
	Debug( "Using device: %s - net: %s - mask: %s - datalink: %s",
	       dev, net, mask, dl_str );
  
	dl_size = dl2off( dl_id );
	Debug ( "datalink header size set at: +%d", dl_size );

	ip_size = 20;
	Debug ( "ip header size set at: +%d", ip_size );

	tcp_size = 20;
	Debug ( "tcp header offset set (temporary) at: +%d", tcp_size );

	first_conn = (conn_t *)S_calloc( sizeof(conn_t), 1 );
	last_conn = first_conn;

	pcap_loop( descr, -1, got_packet, NULL ); 
 
	if ( readfile ) {
		Message( 1, c_DONE, stdout, 
			 "%s: done reading from %s", exe, readfile );
		exit ( TCPICK_SUCCESS );
	}
}

/*
  well the code of tcpick.c is not written very cleanly, i know :|
  I will improve it in the near future (I hope!)
  FIXME
*/
