/*
 * verify.c -- the connection matching engine
 * Part of the tcpick project
 *
 * Author: Francesco Stablum <duskdruid @ despammed.com>
 *
 * Copyright (C) 2003, 2004  Francesco Stablum
 * Licensed under the GPL
 *
 */

/*
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "tcpick.h"
#include "extern.h"

#define ACK_OFF ntohl( tcppacket->ack_seq ) - Desc->oth->sip

#define IS_CLIENT_SENDER \
 current_conn->client.ip   == ippacket->ip_src.s_addr && \
 current_conn->server.ip   == ippacket->ip_dst.s_addr && \
 current_conn->client.port == tcppacket->source       && \
 current_conn->server.port == tcppacket->dest 

#define IS_SERVER_SENDER \
 current_conn->client.ip   == ippacket->ip_dst.s_addr && \
 current_conn->server.ip   == ippacket->ip_src.s_addr && \
 current_conn->client.port == tcppacket->dest         && \
 current_conn->server.port == tcppacket->source 

#define IS_SYN_SENT \
 tcppacket->syn == 1 && \
 tcppacket->ack == 0

#define IS_SYN_RECEIVED \
 tcppacket->syn         == 1        && \
 tcppacket->ack         == 1        && \
 Desc->side             == SERVER   && \
 current_conn->status   == SYN_SENT

#define IS_ESTABLISHING \
 tcppacket->syn           == 0            && \
 tcppacket->ack           == 1            && \
 Desc->side               == CLIENT       && \
 current_conn->status     == SYN_RECEIVED && \
 current_conn->client.sip == ntohl( tcppacket->seq )

#define IS_DATA_FLOW       1

#define IS_FIN_WAIT_1 \
 tcppacket->fin         == 1           && \
 current_conn->status   == ESTABLISHED

#define IS_FIN_WAIT_2__CLOSE_WAIT \
 tcppacket->ack         == 1           && \
 tcppacket->fin         == 0           && \
 current_conn->status   == FIN_WAIT_1  && \
 current_conn->closer   != Desc->side 

#define IS_TIME_WAIT__LAST_ACK \
 tcppacket->ack         == 1                         && \
 tcppacket->fin         == 1                         && \
 ( current_conn->status == FIN_WAIT_2__CLOSE_WAIT ||    \
 current_conn->status == FIN_WAIT_1 )              && \
 current_conn->closer   != Desc->side 

#define IS_CLOSING  \
 tcppacket->ack         == 1                         && \
 tcppacket->fin         == 0                         && \
 current_conn->status   == TIME_WAIT__LAST_ACK       && \
 current_conn->closer   == Desc->side 

extern struct flags_t flags;

extern conn_t * first_conn;
extern conn_t * last_conn;

__inline__ int
established_packet ( conn_t * conn_ptr, struct host_descriptor_t * Desc)
/* called by verify()
   packets of established connections come here */
{
	register u_int32_t off;
	if( payload_len ) {

		off = ntohl( tcppacket->seq ) - Desc->sip ;
		
		/* unacknowledged data */
		addfr( & (Desc->unack),
		       Desc->wlen,
		       off, 
		       payload, 
		       payload_len );
	}
	
     if( ! payload_len || tcppacket->psh ) {
	     /* FIXME: not sure for tcppacket->psh */
	     
	     /* acknowledge data */
	     flush_ack ( Desc, conn_ptr, ACK_OFF  ); 
     }

}

__inline__ int  
verify()
/* called by got_packet().
   this is the engine that matches the packets with their connection */
{

	struct host_descriptor_t * Desc = NULL;

	register conn_t * current_conn;
	
	current_conn = first_conn;
	
/* SYN-SENT: (outside of the loop because the 
 * loop is used for sessions just tracked) */	 
	if( IS_SYN_SENT ) {
		newconn( last_conn ); 
		return(1);
	}
	

	do {
		if ( IS_CLIENT_SENDER ) 
			Desc = & (current_conn->client);
		
		else if ( IS_SERVER_SENDER )
			Desc = & (current_conn->server);
		else {
			current_conn = current_conn->next;
			if(! current_conn) return 0 ;
		}
/* cycle until it finds a connection that 
   matches on server or client side */
	} while ( Desc == NULL );
	
/* when this point is reached, we have found
   the right matching connection */
    
/* FIXME: sucks
	write_banners_and_data(current_conn, 
			       ippacket, tcppacket,
			       payload, payload_len); 
*/

	if( IS_SYN_RECEIVED ) {
		(current_conn)->server.sip = ntohl(tcppacket->seq) + 1;
		status_switch( current_conn, SYN_RECEIVED );
		return 1;
	}
    

/* 3rd packet of the 3-way-handshake*/
	if( IS_ESTABLISHING ) {
		status_switch( current_conn, ESTABLISHED );
		return 1;
	}
    
/* data or ack packets */
	if( IS_DATA_FLOW ) {
		if( REBUILD ) 
			established_packet ( current_conn, Desc );
	}
	
/* first FIN sent */
	if( IS_FIN_WAIT_1 ) {
		status_switch( current_conn, FIN_WAIT_1 );
		(current_conn)->closer = Desc->side; 
		return 1;
	}

     
/* ACK of first FIN sent */
	if( IS_FIN_WAIT_2__CLOSE_WAIT ) {
		status_switch( current_conn, FIN_WAIT_2__CLOSE_WAIT );
		return 1;
	}

     
/* second FIN sent */
	if( IS_TIME_WAIT__LAST_ACK ) {
		status_switch( current_conn, TIME_WAIT__LAST_ACK );
		return 1;
	}

/* LAST-ACK sent */
	if( IS_CLOSING ) {
		status_switch( current_conn, CLOSED );
		return 1;
	}

/* nice work :^) */
}
